
from CvPythonExtensions import *
import CvUtil

class TerraFractal:
	def __init__(self):
		self.gc = CyGlobalContext()
		self.map = self.gc.getMap()
		self.iNumPlotsX = self.map.getGridWidth()
		self.iNumPlotsY = self.map.getGridHeight()
		self.mapRand = self.gc.getGame().getMapRand()
		self.plotTypes = [PlotTypes.PLOT_OCEAN] * (self.iNumPlotsX*self.iNumPlotsY)
		self.fractal = CyTerraFractal()
		# init User Input variances
		self.seaLevelChange = self.gc.getSeaLevelInfo(self.map.getSeaLevel()).getSeaLevelChange()
		self.seaLevelMax = 100
		self.seaLevelMin = 0
		#self.hillGroupOneRange = self.gc.getClimateInfo(self.map.getClimate()).getHillRange()
		#self.hillGroupOneBase = 25
		#self.hillGroupTwoRange = self.gc.getClimateInfo(self.map.getClimate()).getHillRange()
		#self.hillGroupTwoBase = 75
		#self.peakPercent = self.gc.getClimateInfo(self.map.getClimate()).getPeakPercent()
		#self.stripRadius = 15

	def checkForOverrideDefaultUserInputVariances(self):
		# Subclass and override this function to customize/alter/nullify 
		# the XML defaults for user selections on Sea Level, Climate, etc.
		return

	def initFractal(self, width=2, height=1, steps=7, wrapX=true, wrapY=false, polar=true, biasOffset=0):
		self.fractal.setBiasOffset(biasOffset)
		self.fractal.setWrapX(wrapX)
		self.fractal.setWrapY(wrapY)
		self.fractal.setPolar(polar)
		self.fractal.fracInit(width, height, steps);

	def initHintedFractal(self, bitmap="", steps=7, wrapX=true, wrapY=false, polar=true, biasOffset=0):
		self.fractal.setBiasOffset(biasOffset)
		self.fractal.setWrapX(wrapX)
		self.fractal.setWrapY(wrapY)
		self.fractal.setPolar(polar)
		self.fractal.fracInitHinted(bitmap, steps);

	def generatePlotTypes(self, water_percent=78, hills_percent=37, peak_percent=21):
		# Check for changes to User Input variances.
		self.checkForOverrideDefaultUserInputVariances()
		
		water_percent += self.seaLevelChange
		water_percent = min(water_percent, self.seaLevelMax)
		water_percent = max(water_percent, self.seaLevelMin)
		hills_percent = max(0, min(100, hills_percent))
		peak_percent = max(0, min(100, peak_percent))

		water = self.fractal.getHeightFromPercent(water_percent)
		hills = self.fractal.getRampFromPercent(100-hills_percent)
		peak = self.fractal.getRampFromPercent(100-peak_percent)
		#peak = 1.0 - (((1.0 - hills) * peak_percent) / 100)

		for x in range(self.iNumPlotsX):
			for y in range(self.iNumPlotsY):
				i = y*self.iNumPlotsX + x
				height = self.fractal.getHeight(x,y,self.iNumPlotsX,self.iNumPlotsY)
				if height < water:
					self.plotTypes[i] = PlotTypes.PLOT_OCEAN
				else:
					ramp = self.fractal.getRamp(x,y,self.iNumPlotsX,self.iNumPlotsY)
					if (ramp >= peak):
						self.plotTypes[i] = PlotTypes.PLOT_PEAK
					elif (ramp >= hills):
						self.plotTypes[i] = PlotTypes.PLOT_HILLS
					else:
						self.plotTypes[i] = PlotTypes.PLOT_LAND

		return self.plotTypes
